package ch.rakudave.jnetmap.controller;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Arrays;
import java.util.concurrent.TimeUnit;

import ch.rakudave.jnetmap.util.logging.Logger;

/**
 * Detects if another instance is already running and sends it the files to
 * open. If no other instance is running, a ServerSocket is opened to listen for
 * incoming requests.
 * 
 * @author rakudave
 */
public class InstanceDetector {

	public static void delegate(String[] args) {
		try {
			// try to open a socket. If this fails, there's a reasonable chance
			// there's another running instance of 'myself' using the same port.
			final ServerSocket s = new ServerSocket(61337);
			new Thread() {
				@Override
				public void run() {
					while (true) {
						try {
							Socket receive = s.accept();
							String[] sentArgs = (String[]) new ObjectInputStream(receive.getInputStream()).readObject();
							if (sentArgs.length > 0) {
								if ("-close".equals(sentArgs[0])) {
									Scheduler.executor.shutdown();
									receive.close();
									s.close();
									Logger.info("Shutting down...");
									Controller.pm.shutdown();
									Scheduler.executor.awaitTermination(5, TimeUnit.SECONDS);
									System.exit(0);
								} else {
									Controller.open(sentArgs);
								}
							}
						} catch (Exception e) {
							Logger.error("Error in socket communication", e);
						}
					}
				}
			}.start();
		} catch (IOException e) {
			Socket send = null;
			try { // try to send the other instance of myself the files to open and quit
				send = new Socket(InetAddress.getLocalHost(), 61337);
				Logger.info("Found another running instance, sending arguments: " + Arrays.toString(args));
				new ObjectOutputStream(send.getOutputStream()).writeObject(args);
				Logger.info("Successfully delegated, shutting down...");
				System.exit(0);
			} catch (IOException ex) {
				Logger.error("Failed to open output-stream to other instance. Unable to delegate, running own instance...", e);
			} finally {
				try {send.close();} catch (IOException e1) {}
			}
		}
	}

	private InstanceDetector() {
	}
}
