package ch.rakudave.jnetmap.view.jung;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Paint;
import java.awt.Stroke;

import org.apache.commons.collections15.Transformer;

import ch.rakudave.jnetmap.model.Connection;
import ch.rakudave.jnetmap.model.device.Device;
import ch.rakudave.jnetmap.util.Settings;
import edu.uci.ics.jung.visualization.RenderContext;
import edu.uci.ics.jung.visualization.decorators.EdgeShape;
import edu.uci.ics.jung.visualization.picking.PickedInfo;


/**
 * Custom JUNG transformers for edges
 * @author rakudave
 */
public class EdgeTransformers {
	public static enum Shape {Line, Curve, Quad, Bent};

	public static Transformer<Connection, Stroke> strokeTransformer() {
		return new Transformer<Connection, Stroke>() {
			@Override
			public Stroke transform(Connection arg0) {
				float[] dash;
				switch (arg0.getType()) {
					case Coaxial:
						dash = new float[] {20,10,5,10}; break;
					case Fiber:
						dash = new float[] {20,10}; break;
					case Phone:
						dash = new float[] {20,7,2,7}; break;
					case Serial:
						dash = new float[] {20,7,50,7}; break;
					case Wireless:
						dash = new float[] {5,10}; break;
					default:
						dash = new float[] {1}; break;
				}
				return new BasicStroke((float) (Math.log(arg0.getBandwidth())), BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND, 10f, dash, 0f);
			}
		};
	}

	public static Transformer<Connection, Paint> paintTransformer(final PickedInfo<Connection> pi) {
		return new Transformer<Connection, Paint>() {
			@Override
			public Paint transform(Connection arg0) {
				if (pi.isPicked(arg0)) return Color.black;
				return arg0.getStatus().getColor();
			}
			
		};
	}

	public static void setEdgeShape(RenderContext<Device, Connection> rc) {
		switch (Shape.valueOf(Settings.get("edge.shape", "Quad"))) {
			case Line:
				rc.setEdgeShapeTransformer(new EdgeShape.Line<Device, Connection>());
				break;
			case Curve:
				rc.setEdgeShapeTransformer(new EdgeShape.CubicCurve<Device, Connection>());
				break;
			case Bent:
				rc.setEdgeShapeTransformer(new EdgeShape.BentLine<Device, Connection>());
				break;
			default:
				rc.setEdgeShapeTransformer(new EdgeShape.QuadCurve<Device, Connection>());
				break;
		}
	}
}
